-- Author:		Sorcerer Merlin
-- Date:		2012-06-30 @ 23:00 hrs
-- Version:		1.0
-- Desc:		For additional information
--              please contact me on Curse/CurseForge.com.

-- Initialize the Ace3 framework, embedding the AceConsole library in the AddOn
local SMQueue = LibStub("AceAddon-3.0"):NewAddon("SMQueue", "AceConsole-3.0", "AceTimer-3.0");

-- Initialize the timer framework
--local timePassed = 0;

-- Define the EventHandler Frame so we can handle our own events
EventHandler = CreateFrame("Frame", "EventHandler", UIParent);
EventHandler:RegisterEvent("LFG_QUEUE_STATUS_UPDATE");
EventHandler:RegisterEvent("LFG_UPDATE");

-- Define the version of the AddOn
local strVersion = "1.0";

-- Global variables
local teleSafety = false;

-- Static Dialog to confirm leaving LFR raid
StaticPopupDialogs["LEAVE_EARLY"] = {
	text = "The instance is still in progress, are you sure you want to leave?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		LeaveParty()
	end,
	timeout = 30,
	whileDead = true,
	hideOnEscape = true,
}
StaticPopupDialogs["LEAVE_QUEUE"] = {
	text = "Your queue is still in progress, are you sure you want to leave?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		LeaveLFG()
	end,
	timeout = 30,
	whileDead = true,
	hideOnEscape = true,
}

-- Define the LDB plugin itself
local LDB_Lib = LibStub:GetLibrary("LibDataBroker-1.1");
local LDB_plugin = LDB_Lib:NewDataObject("SMQueue", {type = "data source", text = "SMQ", icon = "Interface\\Addons\\SMQueue\\Icons\\EyeBall1"});

-- This function formats time
function formatTime(deltatime)
	output = string.format("%02d:%02d", math.floor(deltatime/60), deltatime-(math.floor(deltatime/60)*60))
	return(output);
end

-- This function is for the timer
function SMQueue:QueueTimer()	
	-- grab the information from the Queue status, and output it
	local hasData,  leaderNeeds, tankNeeds, healerNeeds, dpsNeeds, totalTanks, totalHealers, totalDPS, instanceType, instanceSubType, instanceName, averageWait, tankWait, healerWait, damageWait, myWait, queuedTime = GetLFGQueueStats();
	if hasData then -- we are in the Q
		-- get the time we haved waited, and the estimated time we are supposed to wait and output them.
		lfgWait = formatTime(myWait);			
		--lfgWaited = formatTime(timePassed);
		lfgWaited = formatTime(GetTime() - queuedTime);
		LDB_plugin.text = lfgWaited.."/"..lfgWait;
	else
		LDB_plugin.text = "SMQ"
	end
end

-- This function will be called when the AddOn is initialized by WoW
function SMQueue:OnInitialize()
	-- Output the version of the AddOn we are running
	SMQueue_MsgUser("Version "..strVersion.." loaded!");
end

-- Enable the timer
function SMQueue:OnEnable()
	theTimer = SMQueue:ScheduleRepeatingTimer("QueueTimer", 1);
end

-- This function will be called to send chat messages to the user
function SMQueue_MsgUser(strMessage)
	SMQueue:Print("|cffffd700[Info]: |cffff8c00"..strMessage);
end

-- This function will handle our events (registered at beginning of the AddOn)
EventHandler:SetScript("OnEvent", function(self, event, ...)
	-- do nothing for now
	if event == "LFG_QUEUE_STATUS_UPDATE" then
	
	end
end);

-- Set the actual tooltip that we are going to show here...
function LDB_plugin:OnTooltipShow()	

	-- Get the info we need from the Queue Stats so we can fill out the tooltip
	local hasData,  leaderNeeds, tankNeeds, healerNeeds, dpsNeeds, totalTanks, totalHealers, totalDPS, instanceType, instanceSubType, instanceName, averageWait, tankWait, healerWait, damageWait, myWait, queuedTime = GetLFGQueueStats();

	-- Begin the Tooltip
	GameTooltip:AddLine("SMQueue |cff69CCF0v"..strVersion.."|r");
	GameTooltip:AddLine(" "); -- blank line
	 
	-- check to see if we are in Q before we add more stuff to the tooltip
	if hasData then -- we are in the Q
		name, instanceType, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo()
		-- get the name of the dungeon we are queued for
		GameTooltip:AddLine("Dungeon: |cff69CCF0"..instanceName.."|r");
		
		if (instanceType ~= "party" and instanceType ~= "raid") then -- conceivably we are not yet in a dungeon or raid group, so still in queue
			-- We don't want to print all of this nonsense if we had to add to a party/raid while inside of the instance... will it work?! :P
			-- get the estimated time we are supposed to wait and output them.
			GameTooltip:AddLine("Wait time:  |cff69CCF0"..SecondsToTime(myWait).."|r");
		end
		
		-- get the dungeon roles status/needs
		GameTooltip:AddLine(" "); -- blank line
		-- Tanks first
		if (tankNeeds == 0) then -- we do not need anymore tanks
			GameTooltip:AddLine("Tanks:  |cffABD473"..(totalTanks-tankNeeds).."/"..totalTanks.."|r");
		else -- we are waiting on a tank
			GameTooltip:AddLine("Tanks:  |cffC41F3B"..(totalTanks-tankNeeds).."/"..totalTanks.."|r");
		end
		-- Now DPS second
		if (dpsNeeds == 0) then -- we do not need anymore DPS
			GameTooltip:AddLine("DPS:  |cffABD473"..(totalDPS-dpsNeeds).."/"..totalDPS.."|r");
		else -- we are waiting on a DPS (or more)
			GameTooltip:AddLine("DPS:  |cffC41F3B"..(totalDPS-dpsNeeds).."/"..totalDPS.."|r");
		end
		-- Healers are third
		if (healerNeeds == 0) then -- we do not need anymore heals
			GameTooltip:AddLine("Heals:  |cffABD473"..(totalHealers-healerNeeds).."/"..totalHealers.."|r");
		else -- we are waiting on a heals
			GameTooltip:AddLine("Heals:  |cffC41F3B"..(totalHealers-healerNeeds).."/"..totalHealers.."|r");
		end
		
		-- get the average wait times by role
		
		if (instanceType ~= "party" and instanceType ~= "raid") then -- conceivably we are not yet in a dungeon or raid group, so still in queue
			-- We don't want to print all of this nonsense if we had to add to a party/raid while inside of the instance... will it work?! :P
			GameTooltip:AddLine(" "); -- blank line
			GameTooltip:AddLine("Average wait time for TANK:  |cffFF7D0A"..SecondsToTime(tankWait).."|r");
			GameTooltip:AddLine("Average wait time for DPS:  |cffFF7D0A"..SecondsToTime(damageWait).."|r");
			GameTooltip:AddLine("Average wait time for HEALS:  |cffFF7D0A"..SecondsToTime(healerWait).."|r");
			GameTooltip:AddLine(" "); -- blank line
			GameTooltip:AddLine("|cffFFFFFFLeft-click: |rToggle the queue window.");
			GameTooltip:AddLine("|cffFFFFFFRight-click: |rLeave the queue.");
		else
			-- we are in a dungeon/raid so we need to show teleport and leave party info
			GameTooltip:AddLine(" "); -- blank line
			GameTooltip:AddLine("|cffFFFFFFLeft-click: |rTeleport in/out of instance.");
			GameTooltip:AddLine("|cffFFFFFFRight-click: |rLeave the instance.");
		end
	else
		-- OK we are not in Q, but we may already be in a dungeon/raid
		name, instanceType, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo()
		if (instanceType == "party" or instanceType == "raid") then
			-- we are in a dungeon/raid so we need to show teleport and leave party info
			GameTooltip:AddLine(" "); -- blank line
			GameTooltip:AddLine("|cffFFFFFFLeft-click: |rTeleport in/out of instance.");
			GameTooltip:AddLine("|cffFFFFFFRight-click: |rLeave the instance.");
		else
			-- not in a queue, or already in an instance. show toggle options.
			GameTooltip:AddLine("|cFFFF0000Not in queue.|r");
			GameTooltip:AddLine(" "); -- blank line
			GameTooltip:AddLine("|cffFFFFFFLeft-click: |rToggle the dungeon queue window.");
			GameTooltip:AddLine("|cffFFFFFFRight-click: |rToggle the raid queue window.");
		end
	end
end

-- Display the tooltip
function LDB_plugin:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE");
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT");
	GameTooltip:ClearLines();
	LDB_plugin.OnTooltipShow(GameTooltip);
	GameTooltip:Show();
end

-- Hide the tooltip
function LDB_plugin:OnLeave()
	GameTooltip:Hide();
end

-- The LDB plugin was clicked, do whatever
function LDB_plugin:OnClick(button, down)
	-- Get the info we need from the Queue Stats so we can fill out the tooltip
	local hasData,  leaderNeeds, tankNeeds, healerNeeds, dpsNeeds, totalTanks, totalHealers, totalDPS, instanceType, instanceSubType, instanceName, averageWait, tankWait, healerWait, damageWait, myWait, queuedTime = GetLFGQueueStats();	
	if (hasData) then
		name, instanceType, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo()
		if (instanceType == "party" or instanceType == "raid") then
			-- we are in a dungeon or raid group, now we need to know if we are inside of the instance
			if button == "LeftButton" then
				-- now to check if we need to teleport inside or outside of instance
				isInstance, instanceType = IsInInstance();
				if (isInstance) then
					LFGTeleport(true);
				else
					LFGTeleport(false);
				end
			elseif button == "RightButton" then
				StaticPopup_Show ("LEAVE_EARLY");
			end
		else
			if button == "LeftButton" then -- toggle queue window
				if (instanceSubType == 3) then
					ToggleFrame(RaidParentFrame);
				elseif (instanceSubType == 1) then
					ToggleFrame(LFDParentFrame);
				end
			elseif button == "RightButton" then -- leave the Queue
				StaticPopup_Show ("LEAVE_QUEUE");
			end
		end
	else
		-- OK we are not in Q, but we may already be in a dungeon
		name, instanceType, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo()
		if (instanceType == "party" or instanceType == "raid") then
			-- we are in a dungeon or raid group, now we need to know if we are inside of the instance
			if button == "LeftButton" then
				-- now to check if we need to teleport inside or outside of instance
				isInstance, instanceType = IsInInstance();
				if (isInstance) then
					LFGTeleport(true);
				else
					LFGTeleport(false);
				end
			elseif button == "RightButton" then
				StaticPopup_Show ("LEAVE_EARLY");
			end
		else
			if button == "LeftButton" then -- toggle queue windows
				ToggleFrame(LFDParentFrame);
			elseif button == "RightButton" then
				ToggleFrame(RaidParentFrame);
			end
		end
	end
end
